@@ -40,6 +40,11 @@ local hstore_match_only = false
 -- hstore column.
 local hstore_column = nil
 
+-- If this is set, area calculations are always in Mercator coordinates units
+-- irrespective of the srid setting.
+-- Set this to true if you used --reproject-area before.
+local reproject_area = false
+
 -- There is some very old specialized handling of route relations in osm2pgsql,
 -- which you probably don't need. This is disabled here, but you can enable
 -- it by setting this to true. If you don't understand this, leave it alone.
@@ -52,9 +57,8 @@ if hstore and hstore_all then
 end
 
 -- Used for splitting up long linestrings
-if srid == 4326 then
-    max_length = 1
-else
+local max_length = 1
+if srid == 3857 then
     max_length = 100000
 end
 
@@ -320,8 +324,8 @@ local non_point_columns = {
     'wood',
 }
 
-function gen_columns(text_columns, with_hstore, area, geometry_type)
-    columns = {}
+local function gen_columns(text_columns, with_hstore, area, geometry_type)
+    local columns = {}
 
     local add_column = function (name, type)
         columns[#columns + 1] = { column = name, type = type }
@@ -333,12 +337,8 @@ function gen_columns(text_columns, with_hstore, area, geometry_type)
 
     add_column('z_order', 'int')
 
-    if area ~= nil then
-        if area then
-            add_column('way_area', 'area')
-        else
-            add_column('way_area', 'real')
-        end
+    if area then
+        add_column('way_area', 'real')
     end
 
     if hstore_column then
@@ -361,13 +361,13 @@ local tables = {}
 tables.point = osm2pgsql.define_table{
     name = prefix .. '_point',
     ids = { type = 'node', id_column = 'osm_id' },
-    columns = gen_columns(point_columns, hstore or hstore_all, nil, 'point')
+    columns = gen_columns(point_columns, hstore or hstore_all, false, 'point')
 }
 
 tables.line = osm2pgsql.define_table{
     name = prefix .. '_line',
     ids = { type = 'way', id_column = 'osm_id' },
-    columns = gen_columns(non_point_columns, hstore or hstore_all, false, 'linestring')
+    columns = gen_columns(non_point_columns, hstore or hstore_all, true, 'linestring')
 }
 
 tables.polygon = osm2pgsql.define_table{
@@ -379,7 +379,7 @@ tables.polygon = osm2pgsql.define_table{
 tables.roads = osm2pgsql.define_table{
     name = prefix .. '_roads',
     ids = { type = 'way', id_column = 'osm_id' },
-    columns = gen_columns(non_point_columns, hstore or hstore_all, false, 'linestring')
+    columns = gen_columns(non_point_columns, hstore or hstore_all, true, 'linestring')
 }
 
 local z_order_lookup = {
@@ -412,11 +412,11 @@ local z_order_lookup = {
     motorway = {39, true}
 }
 
-function as_bool(value)
+local function as_bool(value)
     return value == 'yes' or value == 'true' or value == '1'
 end
 
-function get_z_order(tags)
+local function get_z_order(tags)
     local z_order = 100 * math.floor(tonumber(tags.layer or '0') or 0)
     local roads = false
 
@@ -447,7 +447,7 @@ function get_z_order(tags)
     return z_order, roads
 end
 
-function make_check_in_list_func(list)
+local function make_check_in_list_func(list)
     local h = {}
     for _, k in ipairs(list) do
         h[k] = true
@@ -465,7 +465,7 @@ end
 local is_polygon = make_check_in_list_func(polygon_keys)
 local clean_tags = osm2pgsql.make_clean_tags_func(delete_keys)
 
-function make_column_hash(columns)
+local function make_column_hash(columns)
     local h = {}
 
     for _, k in ipairs(columns) do
@@ -475,7 +475,7 @@ function make_column_hash(columns)
     return h
 end
 
-function make_get_output(columns, hstore_all)
+local function make_get_output(columns)
     local h = make_column_hash(columns)
     if hstore_all then
         return function(tags)
@@ -511,10 +511,10 @@ end
 
 local has_generic_tag = make_check_in_list_func(generic_keys)
 
-local get_point_output = make_get_output(point_columns, hstore_all)
-local get_non_point_output = make_get_output(non_point_columns, hstore_all)
+local get_point_output = make_get_output(point_columns)
+local get_non_point_output = make_get_output(non_point_columns)
 
-function get_hstore_column(tags)
+local function get_hstore_column(tags)
     local len = #hstore_column
     local h = {}
     for k, v in pairs(tags) do
@@ -561,7 +561,7 @@ function osm2pgsql.process_node(object)
     tables.point:insert(output)
 end
 
-function add_line(output, geom, roads)
+local function add_line(output, geom, roads)
     for sgeom in geom:segmentize(max_length):geometries() do
         output.way = sgeom
         tables.line:insert(output)
@@ -571,6 +571,20 @@ function add_line(output, geom, roads)
     end
 end
 
+local function compute_geom_and_area(geom)
+    local area, projected_geom
+
+    projected_geom = geom:transform(srid)
+
+    if reproject_area and srid ~= 3857 then
+        area = geom:transform(3857):area()
+    else
+        area = projected_geom:area()
+    end
+
+    return projected_geom, area
+end
+
 function osm2pgsql.process_way(object)
     if clean_tags(object.tags) then
         return
@@ -629,7 +643,9 @@ function osm2pgsql.process_way(object)
     end
 
     if polygon and object.is_closed then
-        output.way = object:as_polygon()
+        local pgeom, area = compute_geom_and_area(object:as_polygon())
+        output.way = pgeom
+        output.way_area = area
         tables.polygon:insert(output)
     else
         add_line(output, object:as_linestring(), roads)
@@ -731,12 +747,17 @@ function osm2pgsql.process_relation(object)
 
     if make_boundary or make_polygon then
         local geom = object:as_multipolygon()
+
         if multi_geometry then
-            output.way = geom
+            local pgeom, area = compute_geom_and_area(geom)
+            output.way = pgeom
+            output.way_area = area
             tables.polygon:insert(output)
         else
             for sgeom in geom:geometries() do
-                output.way = sgeom
+                local pgeom, area = compute_geom_and_area(sgeom)
+                output.way = pgeom
+                output.way_area = area
                 tables.polygon:insert(output)
             end
         end
