@@ -6,10 +6,11 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
+#include <cassert>
 #include <condition_variable>
 #include <deque>
 #include <future>
@@ -26,31 +27,44 @@
 /**
  * Table information necessary for building SQL queries.
  */
-struct db_target_descr_t
+class db_target_descr_t
 {
-    /// Schema of the target table (can be empty for default schema)
-    std::string schema;
-    /// Name of the target table for the copy operation.
-    std::string name;
-    /// Name of id column used when deleting objects.
-    std::string id;
-    /// Comma-separated list of rows for copy operation (when empty: all rows)
-    std::string rows;
+public:
+    db_target_descr_t(std::string schema, std::string name, std::string id,
+                      std::string rows = {})
+    : m_schema(std::move(schema)), m_name(std::move(name)), m_id(std::move(id)),
+      m_rows(std::move(rows))
+    {
+        assert(!m_schema.empty());
+        assert(!m_name.empty());
+    }
+
+    std::string const &schema() const noexcept { return m_schema; }
+    std::string const &name() const noexcept { return m_name; }
+    std::string const &id() const noexcept { return m_id; }
+    std::string const &rows() const noexcept { return m_rows; }
+
+    void set_rows(std::string rows) { m_rows = std::move(rows); }
 
     /**
      * Check if the buffer would use exactly the same copy operation.
      */
     bool same_copy_target(db_target_descr_t const &other) const noexcept
     {
-        return (this == &other) || (schema == other.schema &&
-                                    name == other.name && rows == other.rows);
+        return (this == &other) ||
+               (m_schema == other.m_schema && m_name == other.m_name &&
+                m_id == other.m_id && m_rows == other.m_rows);
     }
 
-    db_target_descr_t() = default;
-
-    db_target_descr_t(std::string n, std::string i, std::string r = {})
-    : name(std::move(n)), id(std::move(i)), rows(std::move(r))
-    {}
+private:
+    /// Schema of the target table.
+    std::string m_schema;
+    /// Name of the target table for the copy operation.
+    std::string m_name;
+    /// Name of id column used when deleting objects.
+    std::string m_id;
+    /// Comma-separated list of rows for copy operation (when empty: all rows)
+    std::string m_rows;
 };
 
 /**
@@ -198,8 +212,9 @@ public:
     void delete_data(pg_conn_t *conn) override
     {
         if (m_deleter.has_data()) {
-            m_deleter.delete_rows(qualified_name(target->schema, target->name),
-                                  target->id, conn);
+            m_deleter.delete_rows(
+                qualified_name(target->schema(), target->name()), target->id(),
+                conn);
         }
     }
 
@@ -234,7 +249,7 @@ struct db_cmd_finish_t : public db_cmd_t
 class db_copy_thread_t
 {
 public:
-    explicit db_copy_thread_t(std::string const &conninfo);
+    explicit db_copy_thread_t(connection_params_t const &connection_params);
 
     db_copy_thread_t(db_copy_thread_t const &) = delete;
     db_copy_thread_t &operator=(db_copy_thread_t const &) = delete;
@@ -275,7 +290,7 @@ private:
     class thread_t
     {
     public:
-        thread_t(std::string conninfo, shared *shared);
+        thread_t(connection_params_t connection_params, shared *shared);
 
         void operator()();
 
@@ -285,7 +300,7 @@ private:
         void finish_copy();
         void delete_rows(db_cmd_copy_t *buffer);
 
-        std::string m_conninfo;
+        connection_params_t m_connection_params;
         std::unique_ptr<pg_conn_t> m_conn;
 
         // Target for copy operation currently ongoing.
