@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -16,6 +16,7 @@
 #include "pgsql.hpp"
 #include "reprojection.hpp"
 #include "thread-pool.hpp"
+#include "util.hpp"
 
 #include <osmium/osm/item_type.hpp>
 
@@ -26,6 +27,21 @@
 #include <utility>
 #include <vector>
 
+/**
+ * This defines the type of "primary key" for the tables generated in the flex
+ * output. This is not a real primary key, because the values are not
+ * necessarily unique.
+ */
+enum class flex_table_index_type {
+    no_index,
+    node, // index by node id
+    way, // index by way id
+    relation, // index by relation id
+    area, // index by way (positive) or relation (negative) id
+    any_object, // any OSM object, two columns for type and id
+    tile // index by tile with x and y columns (used for generalized data)
+};
+
 /**
  * An output table (in the SQL sense) for the flex backend.
  */
@@ -43,7 +59,10 @@ public:
         permanent
     };
 
-    explicit flex_table_t(std::string name) : m_name(std::move(name)) {}
+    flex_table_t(std::string schema, std::string name, std::size_t num)
+    : m_schema(std::move(schema)), m_name(std::move(name)), m_table_num(num)
+    {
+    }
 
     std::string const &name() const noexcept { return m_name; }
 
@@ -84,9 +103,9 @@ public:
         m_index_tablespace = std::move(tablespace);
     }
 
-    osmium::item_type id_type() const noexcept { return m_id_type; }
+    flex_table_index_type id_type() const noexcept { return m_id_type; }
 
-    void set_id_type(osmium::item_type type) noexcept { m_id_type = type; }
+    void set_id_type(flex_table_index_type type) noexcept { m_id_type = type; }
 
     bool has_id_column() const noexcept;
 
@@ -102,6 +121,11 @@ public:
         return m_columns.end();
     }
 
+    flex_table_column_t *find_column_by_name(std::string const &name)
+    {
+        return util::find_by_name(m_columns, name);
+    }
+
     bool has_geom_column() const noexcept
     {
         return m_geom_column != std::numeric_limits<std::size_t>::max();
@@ -114,6 +138,12 @@ public:
         return m_columns[m_geom_column];
     }
 
+    flex_table_column_t &geom_column() noexcept
+    {
+        assert(has_geom_column());
+        return m_columns[m_geom_column];
+    }
+
     int srid() const noexcept
     {
         return has_geom_column() ? geom_column().srid() : 4326;
@@ -165,13 +195,17 @@ public:
         return m_always_build_id_index;
     }
 
+    bool has_columns_with_expire() const noexcept;
+
+    std::size_t num() const noexcept { return m_table_num; }
+
 private:
+    /// The schema this table is in
+    std::string m_schema;
+
     /// The name of the table
     std::string m_name;
 
-    /// The schema this table is in
-    std::string m_schema{"public"};
-
     /// The table space used for this table (empty for default tablespace)
     std::string m_data_tablespace;
 
@@ -198,11 +232,13 @@ private:
      */
     std::size_t m_geom_column = std::numeric_limits<std::size_t>::max();
 
+    /// Unique number for each table.
+    std::size_t m_table_num;
+
     /**
-     * Type of Id stored in this table (node, way, relation, area, or
-     * undefined for any type).
+     * Type of id stored in this table.
      */
-    osmium::item_type m_id_type = osmium::item_type::undefined;
+    flex_table_index_type m_id_type = flex_table_index_type::no_index;
 
     /// Cluster the table by geometry.
     bool m_cluster_by_geom = true;
@@ -222,30 +258,32 @@ public:
                        std::shared_ptr<db_copy_thread_t> const &copy_thread)
     : m_proj(reprojection::create_projection(table->srid())), m_table(table),
       m_target(std::make_shared<db_target_descr_t>(
-          table->name(), table->id_column_names(),
+          table->schema(), table->name(), table->id_column_names(),
           table->build_sql_column_list())),
-      m_copy_mgr(copy_thread), m_db_connection(nullptr)
+      m_copy_mgr(copy_thread)
     {
-        m_target->schema = table->schema();
     }
 
-    void connect(std::string const &conninfo);
+    void start(pg_conn_t const &db_connection, bool append);
 
-    void start(bool append);
-
-    void stop(bool updateable, bool append);
+    void stop(pg_conn_t const &db_connection, bool updateable, bool append);
 
     flex_table_t const &table() const noexcept { return *m_table; }
 
-    void teardown() { m_db_connection.reset(); }
+    void prepare(pg_conn_t const &db_connection);
 
-    void prepare();
+    void analyze(pg_conn_t const &db_connection);
 
-    void analyze();
+    void create_id_index(pg_conn_t const &db_connection);
 
-    void create_id_index();
+    /**
+     * Get all geometries that have at least one expire config defined
+     * from the database and return the result set.
+     */
+    pg_result_t get_geoms_by_id(pg_conn_t const &db_connection,
+                                osmium::item_type type, osmid_t id) const;
 
-    pg_result_t get_geom_by_id(osmium::item_type type, osmid_t id) const;
+    void flush() { m_copy_mgr.flush(); }
 
     void sync() { m_copy_mgr.sync(); }
 
@@ -271,6 +309,13 @@ public:
 
     void task_wait();
 
+    void increment_insert_counter() noexcept { ++m_count_insert; }
+
+    void increment_not_null_error_counter() noexcept
+    {
+        ++m_count_not_null_error;
+    }
+
 private:
     std::shared_ptr<reprojection> m_proj;
 
@@ -284,11 +329,11 @@ private:
      */
     db_copy_mgr_t<db_deleter_by_type_and_id_t> m_copy_mgr;
 
-    /// The connection to the database server.
-    std::unique_ptr<pg_conn_t> m_db_connection;
-
     task_result_t m_task_result;
 
+    std::size_t m_count_insert = 0;
+    std::size_t m_count_not_null_error = 0;
+
     /// Has the Id index already been created?
     bool m_id_index_created = false;
 
