@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -16,6 +16,7 @@
  */
 
 #include "canvas.hpp"
+#include "command-line-app.hpp"
 #include "debug-output.hpp"
 #include "expire-output.hpp"
 #include "flex-lua-expire-output.hpp"
@@ -78,59 +79,6 @@ constexpr std::size_t const max_force_single_thread = 4;
         }                                                                      \
     }
 
-static void show_help()
-{
-    fmt::print(R"(osm2pgsql-gen [OPTIONS]
-Generalization of OSM data.
-
-This program is EXPERIMENTAL and might change without notice.
-
-Main Options:
-    -a|--append           Run in append mode
-    -c|--create           Run in create mode (default)
-    -S|--style=FILE       The Lua config file (same as for osm2pgsql)
-    -j|--jobs=NUM         Number of parallel jobs (default 1)
-       --middle-schema=SCHEMA  Database schema for middle tables (default set with --schema)
-       --schema=SCHEMA    Default database schema (default: 'public')
-
-Help/Version Options:
-    -h|--help             Print this help text and stop
-    -V|--version          Show version
-
-Logging options:
-    -l|--log-level=LEVEL  Log level (debug, info (default), warn, error)
-       --log-sql          Log SQL commands
-
-Database options:
-    -d|--database=DB    The name of the PostgreSQL database to connect to or
-                        a PostgreSQL conninfo string.
-    -U|--username=NAME  PostgreSQL user name.
-    -W|--password       Force password prompt.
-    -H|--host=HOST      Database server host name or socket location.
-    -P|--port=PORT      Database server port.
-)");
-}
-
-static char const *const short_options = "acd:hH:j:l:P:S:U:VW";
-
-static std::array<option, 20> const long_options = {
-    {{"help", no_argument, nullptr, 'h'},
-     {"version", no_argument, nullptr, 'V'},
-     {"append", no_argument, nullptr, 'a'},
-     {"create", no_argument, nullptr, 'c'},
-     {"jobs", required_argument, nullptr, 'j'},
-     {"database", required_argument, nullptr, 'd'},
-     {"user", required_argument, nullptr, 'U'},
-     {"host", required_argument, nullptr, 'H'},
-     {"port", required_argument, nullptr, 'P'},
-     {"password", no_argument, nullptr, 'W'},
-     {"log-level", required_argument, nullptr, 'l'},
-     {"style", required_argument, nullptr, 'S'},
-     {"log-sql", no_argument, nullptr, 201},
-     {"middle-schema", required_argument, nullptr, 202},
-     {"schema", required_argument, nullptr, 203},
-     {nullptr, 0, nullptr, 0}}};
-
 struct tile_extent
 {
     uint32_t xmin = 0;
@@ -200,21 +148,31 @@ static tile_extent get_extent_from_db(pg_conn_t const &db_connection,
     return get_extent_from_db(db_connection, schema, table, geom_column, zoom);
 }
 
-static std::vector<std::pair<uint32_t, uint32_t>>
-get_tiles_from_table(pg_conn_t const &connection, std::string const &table)
+static void
+get_tiles_from_table(pg_conn_t const &connection, std::string const &table,
+                     uint32_t zoom,
+                     std::vector<std::pair<uint32_t, uint32_t>> *tiles)
 {
-    std::vector<std::pair<uint32_t, uint32_t>> tiles;
+    auto const result = connection.exec(
+        R"(SELECT x, y FROM "{}" WHERE zoom = {})", table, zoom);
 
-    auto const result = connection.exec(R"(SELECT x, y FROM "{}")", table);
+    tiles->reserve(result.num_tuples());
 
+    uint32_t const max = 1UL << zoom;
     for (int n = 0; n < result.num_tuples(); ++n) {
         char *end = nullptr;
         auto const x = std::strtoul(result.get_value(n, 0), &end, 10);
+        if (*end != '\0' || x >= max) {
+            log_error("Ignoring invalid x value in expire table '{}'", table);
+            continue;
+        }
         auto const y = std::strtoul(result.get_value(n, 1), &end, 10);
-        tiles.emplace_back(x, y);
+        if (*end != '\0' || y >= max) {
+            log_error("Ignoring invalid y value in expire table '{}'", table);
+            continue;
+        }
+        tiles->emplace_back(x, y);
     }
-
-    return tiles;
 }
 
 class tile_processor_t
@@ -237,8 +195,9 @@ private:
     std::size_t m_num_tiles;
 };
 
-void run_tile_gen(std::string const &conninfo, gen_base_t *master_generalizer,
-                  params_t params, uint32_t zoom,
+void run_tile_gen(connection_params_t const &connection_params,
+                  gen_base_t *master_generalizer, params_t params,
+                  uint32_t zoom,
                   std::vector<std::pair<uint32_t, uint32_t>> *queue,
                   std::mutex *mut, unsigned int n)
 {
@@ -246,7 +205,7 @@ void run_tile_gen(std::string const &conninfo, gen_base_t *master_generalizer,
 
     log_debug("Started generalizer thread for '{}'.",
               master_generalizer->strategy());
-    pg_conn_t db_connection{conninfo};
+    pg_conn_t db_connection{connection_params, "gen.tile"};
     std::string const strategy{master_generalizer->strategy()};
     auto generalizer = create_generalizer(strategy, &db_connection, &params);
 
@@ -262,7 +221,10 @@ void run_tile_gen(std::string const &conninfo, gen_base_t *master_generalizer,
             queue->pop_back();
         }
 
-        generalizer->process({zoom, p.first, p.second});
+        tile_t const tile{zoom, p.first, p.second};
+        log_debug("Processing tile {}/{}/{}...", tile.zoom(), tile.x(),
+                  tile.y());
+        generalizer->process(tile);
     }
     log_debug("Shutting down generalizer thread.");
 }
@@ -270,9 +232,9 @@ void run_tile_gen(std::string const &conninfo, gen_base_t *master_generalizer,
 class genproc_t
 {
 public:
-    genproc_t(std::string const &filename, std::string conninfo,
-              std::string dbschema, bool append, bool updatable,
-              uint32_t jobs);
+    genproc_t(std::string const &filename,
+              connection_params_t connection_params, std::string dbschema,
+              bool append, bool updatable, uint32_t jobs);
 
     int app_define_table()
     {
@@ -296,7 +258,7 @@ public:
 
     int app_run_gen()
     {
-        log_debug("Running configured generalizer (run {})...", ++m_gen_run);
+        log_debug("Configuring generalizer...");
 
         if (lua_type(lua_state(), 1) != LUA_TSTRING) {
             throw std::runtime_error{"Argument #1 to 'run_gen' must be a "
@@ -317,24 +279,25 @@ public:
             params.set("schema", m_dbschema);
         }
 
+        if (m_append) {
+            params.set("delete_existing", true);
+        }
+
         write_to_debug_log(params, "Params (config):");
 
         log_debug("Connecting to database...");
-        pg_conn_t db_connection{m_conninfo};
+        pg_conn_t db_connection{m_connection_params, "gen.proc"};
 
         log_debug("Creating generalizer...");
         auto generalizer =
             create_generalizer(strategy, &db_connection, &params);
 
-        log_debug("Generalizer '{}' ({}) initialized.", generalizer->name(),
-                  generalizer->strategy());
-
-        if (m_append) {
-            params.set("delete_existing", true);
-        }
+        log_info("Running generalizer '{}' ({})...", generalizer->name(),
+                 generalizer->strategy());
 
         write_to_debug_log(params, "Params (after initialization):");
 
+        util::timer_t timer_gen;
         if (generalizer->on_tiles()) {
             process_tiles(db_connection, params, generalizer.get());
         } else {
@@ -353,8 +316,8 @@ public:
                 std::chrono::duration_cast<std::chrono::milliseconds>(
                     timer.elapsed())));
         }
-        log_debug("Finished generalizer '{}' (run {}).", generalizer->name(),
-                  m_gen_run);
+        log_info("Finished generalizer '{}' in {}.", generalizer->name(),
+                 util::human_readable_duration(timer_gen.stop()));
 
         return 0;
     }
@@ -368,16 +331,64 @@ public:
 
         std::string const description =
             luaX_get_table_string(lua_state(), "description", 1, "Argument #1");
-        std::string const sql =
-            luaX_get_table_string(lua_state(), "sql", 1, "Argument #1");
 
-        log_debug("Running SQL command: {}.", description);
+        bool const transaction = luaX_get_table_bool(lua_state(), "transaction",
+                                                     1, "Argument #1", false);
+
+        std::string const if_has_rows = luaX_get_table_string(
+            lua_state(), "if_has_rows", 1, "Argument #1", "");
+
+        std::vector<std::string> queries;
+        if (transaction) {
+            queries.emplace_back("BEGIN");
+        }
+
+        lua_getfield(lua_state(), 1, "sql");
+        int const ltype = lua_type(lua_state(), -1);
+        if (ltype == LUA_TSTRING) {
+            queries.emplace_back(lua_tostring(lua_state(), -1));
+        } else if (ltype == LUA_TTABLE) {
+            if (!luaX_is_array(lua_state())) {
+                throw std::runtime_error{
+                    "Table in 'sql' field must be an array."};
+            }
+            luaX_for_each(lua_state(), [&]() {
+                if (lua_type(lua_state(), -1) != LUA_TSTRING) {
+                    throw std::runtime_error{
+                        "Table in 'sql' field must only contain strings."};
+                }
+                queries.emplace_back(lua_tostring(lua_state(), -1));
+            });
+        } else {
+            throw std::runtime_error{
+                "Argument #1 must contain a 'sql' string or table field."};
+        }
+
+        if (transaction) {
+            queries.emplace_back("COMMIT");
+        }
+
+        pg_conn_t const db_connection{m_connection_params, "gen.sql"};
+
+        if (m_append && !if_has_rows.empty()) {
+            auto const result = db_connection.exec(if_has_rows);
+            if (result.num_tuples() == 0) {
+                log_info("Not running SQL command: {} (no rows in "
+                         "condition result).",
+                         description);
+                return 0;
+            }
+        }
+
+        log_info("Running SQL commands: {}.", description);
 
         util::timer_t timer_sql;
-        pg_conn_t const db_connection{m_conninfo};
-        db_connection.exec(sql);
-        log_debug("SQL command took {}.",
-                  util::human_readable_duration(timer_sql.stop()));
+        for (auto const &query : queries) {
+            log_debug("Running sql: {}", query);
+            db_connection.exec(query);
+        }
+        log_info("Finished SQL commands in {}.",
+                 util::human_readable_duration(timer_sql.stop()));
 
         return 0;
     }
@@ -445,7 +456,7 @@ private:
             auto const table = params.get_string("expire_list");
             log_debug("Running generalizer for expire list from table '{}'...",
                       table);
-            tile_list = get_tiles_from_table(db_connection, table);
+            get_tiles_from_table(db_connection, table, zoom, &tile_list);
             log_debug("Truncating table '{}'...", table);
             db_connection.exec("TRUNCATE {}", table);
         } else {
@@ -485,8 +496,9 @@ private:
             for (unsigned int n = 1;
                  n <= std::min(m_jobs, static_cast<uint32_t>(tile_list.size()));
                  ++n) {
-                threads.emplace_back(run_tile_gen, m_conninfo, generalizer,
-                                     params, zoom, &tile_list, &mut, n);
+                threads.emplace_back(run_tile_gen, m_connection_params,
+                                     generalizer, params, zoom, &tile_list,
+                                     &mut, n);
             }
             for (auto &t : threads) {
                 t.join();
@@ -502,9 +514,8 @@ private:
     std::vector<flex_table_t> m_tables;
     std::vector<expire_output_t> m_expire_outputs;
 
-    std::string m_conninfo;
+    connection_params_t m_connection_params;
     std::string m_dbschema;
-    std::size_t m_gen_run = 0;
     uint32_t m_jobs;
     bool m_append;
     bool m_updatable;
@@ -515,11 +526,13 @@ TRAMPOLINE(app_define_expire_output, define_expire_output)
 TRAMPOLINE(app_run_gen, run_gen)
 TRAMPOLINE(app_run_sql, run_sql)
 
-genproc_t::genproc_t(std::string const &filename, std::string conninfo,
+genproc_t::genproc_t(std::string const &filename,
+                     connection_params_t connection_params,
                      std::string dbschema, bool append, bool updatable,
                      uint32_t jobs)
-: m_conninfo(std::move(conninfo)), m_dbschema(std::move(dbschema)),
-  m_jobs(jobs), m_append(append), m_updatable(updatable)
+: m_connection_params(std::move(connection_params)),
+  m_dbschema(std::move(dbschema)), m_jobs(jobs), m_append(append),
+  m_updatable(updatable)
 {
     setup_lua_environment(lua_state(), filename, append);
 
@@ -579,7 +592,7 @@ void genproc_t::run()
     }
 
     if (!m_append) {
-        pg_conn_t const db_connection{m_conninfo};
+        pg_conn_t const db_connection{m_connection_params, "gen.index"};
         for (auto const &table : m_tables) {
             if (table.id_type() == flex_table_index_type::tile &&
                 (table.always_build_id_index() || m_updatable)) {
@@ -595,115 +608,82 @@ void genproc_t::run()
     }
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 int main(int argc, char *argv[])
 {
     try {
-        database_options_t database_options;
         std::string dbschema{"public"};
-        std::string middle_dbschema{};
-        std::string log_level;
+        std::string middle_dbschema;
         std::string style;
         uint32_t jobs = 1;
-        bool pass_prompt = false;
         bool append = false;
 
-        int c = 0;
-        // NOLINTNEXTLINE(concurrency-mt-unsafe)
-        while (-1 != (c = getopt_long(argc, argv, short_options,
-                                      long_options.data(), nullptr))) {
-            switch (c) {
-            case 'h': // --help
-                show_help();
-                return 0;
-            case 'a': // --append
-                append = true;
-                break;
-            case 'c': // --create
-                append = false;
-                break;
-            case 'j': // --jobs
-                jobs =
-                    std::clamp(std::strtoul(optarg, nullptr, 10), 1UL, 256UL);
-                break;
-            case 'd': // --database
-                database_options.db = optarg;
-                break;
-            case 'U': // --username
-                database_options.username = optarg;
-                break;
-            case 'W': // --password
-                pass_prompt = true;
-                break;
-            case 'H': // --host
-                database_options.host = optarg;
-                break;
-            case 'P': // --port
-                database_options.port = optarg;
-                break;
-            case 'l': // --log-level
-                log_level = optarg;
-                break;
-            case 'S': // --style
-                style = optarg;
-                break;
-            case 'V': // --version
-                log_info("osm2pgsql-gen version {}", get_osm2pgsql_version());
-                canvas_t::info();
-                return 0;
-            case 201: // --log-sql
-                get_logger().enable_sql();
-                break;
-            case 202: // --middle-schema
-                middle_dbschema = optarg;
-                if (middle_dbschema.empty()) {
-                    log_error("Schema must not be empty");
-                    return 2;
-                }
-                check_identifier(middle_dbschema, "--middle-schema");
-                break;
-            case 203: // --schema
-                dbschema = optarg;
-                if (dbschema.empty()) {
-                    log_error("Schema must not be empty");
-                    return 2;
-                }
-                check_identifier(dbschema, "--schema");
-                break;
-            default:
-                log_error("Unknown argument");
-                return 2;
-            }
+        command_line_app_t app{
+            "osm2pgsql-gen -- Generalize OpenStreetMap data\n"};
+
+        // ------------------------------------------------------------------
+        // Main options
+        // ------------------------------------------------------------------
+
+        app.add_flag("-a,--append", append)->description("Run in append mode.");
+
+        app.add_option("-S,--style", style)
+            ->description("The Lua config/style file (same as for osm2pgsql).")
+            ->type_name("FILE");
+
+        app.add_option("-j,--jobs", jobs)
+            ->check(CLI::Range(1, 256))
+            ->description("Number of parallel jobs (default: 1, max 256).")
+            ->type_name("NUM");
+
+        // ------------------------------------------------------------------
+        // Database options
+        // ------------------------------------------------------------------
+
+        app.add_option("--middle-schema", middle_dbschema)
+            ->description("Database schema for middle tables (default: setting "
+                          "of --schema).")
+            ->type_name("SCHEMA")
+            ->group("Database options");
+
+        app.add_option("--schema", dbschema)
+            ->description("Database schema (default: 'public').")
+            ->type_name("SCHEMA")
+            ->group("Database options");
+
+        try {
+            app.parse(argc, argv);
+        } catch (...) {
+            log_info("osm2pgsql-gen version {}", get_osm2pgsql_version());
+            throw;
         }
 
-        if (log_level == "debug") {
-            get_logger().set_level(log_level::debug);
-        } else if (log_level == "info" || log_level.empty()) {
-            get_logger().set_level(log_level::info);
-        } else if (log_level == "warn") {
-            get_logger().set_level(log_level::warn);
-        } else if (log_level == "error") {
-            get_logger().set_level(log_level::error);
-        } else {
-            log_error("Unknown log level: {}. "
-                      "Use 'debug', 'info', 'warn', or 'error'.",
-                      log_level);
-            return 2;
+        if (app.want_help()) {
+            std::cout << app.help();
+            return 0;
         }
 
-        if (jobs < 1 || jobs > 32) {
-            log_error("The --jobs/-j parameter must be between 1 and 32.");
-            return 2;
-        }
-
-        if (middle_dbschema.empty()) {
-            middle_dbschema = dbschema;
-        }
-
-        util::timer_t timer_overall;
-
         log_info("osm2pgsql-gen version {}", get_osm2pgsql_version());
         log_warn("This is an EXPERIMENTAL extension to osm2pgsql.");
 
+        if (app.want_version()) {
+            return 0;
+        }
+
+        if (dbschema.empty()) {
+            log_error("Schema must not be empty");
+            return 2;
+        }
+        check_identifier(dbschema, "--schema");
+
+        if (middle_dbschema.empty()) {
+            middle_dbschema = dbschema;
+        } else {
+            check_identifier(middle_dbschema, "--middle-schema");
+        }
+
+        util::timer_t timer_overall;
+
         if (append) {
             log_debug("Running in append mode.");
         } else {
@@ -718,18 +698,15 @@ int main(int argc, char *argv[])
                 jobs);
         }
 
-        if (pass_prompt) {
-            database_options.password = util::get_password();
-        }
-        auto const conninfo = build_conninfo(database_options);
+        auto const connection_params = app.connection_params();
 
         log_debug("Checking database capabilities...");
         {
-            pg_conn_t const db_connection{conninfo};
+            pg_conn_t const db_connection{connection_params, "gen.check"};
             init_database_capabilities(db_connection);
         }
 
-        properties_t properties{conninfo, middle_dbschema};
+        properties_t properties{connection_params, middle_dbschema};
         properties.load();
 
         if (style.empty()) {
@@ -740,8 +717,14 @@ int main(int argc, char *argv[])
             }
         }
 
+        if (properties.get_string("output", "flex") != "flex") {
+            throw std::runtime_error{
+                "osm2pgsql-gen only works with flex output"};
+        }
+
         bool const updatable = properties.get_bool("updatable", false);
-        genproc_t gen{style, conninfo, dbschema, append, updatable, jobs};
+        genproc_t gen{style,  connection_params, dbschema,
+                      append, updatable,         jobs};
         gen.run();
 
         osmium::MemoryUsage const mem;
