@@ -6,10 +6,12 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
+#include "pgsql-params.hpp"
+
 #include <osmium/osm/box.hpp>
 
 #include <cstdint>
@@ -37,43 +39,6 @@ enum class hstore_column : char
     all = 2
 };
 
-/// Database connection options.
-struct database_options_t
-{
-    std::string db;
-    std::string username;
-    std::string host;
-    std::string password;
-    std::string port;
-};
-
-std::string build_conninfo(database_options_t const &opt);
-
-/**
- * Outputs can signal their requirements to the middle by setting these fields.
- */
-struct output_requirements
-{
-    /**
-     * Need full node objects with tags, attributes (only if --extra-attributes
-     * is set) and locations. If false, only node locations are needed.
-     */
-    bool full_nodes = false;
-
-    /**
-     * Need full way objects with tags, attributes (only if --extra-attributes
-     * is set) and way nodes. If false, only way nodes are needed.
-     */
-    bool full_ways = false;
-
-    /**
-     * Need full relation objects with tags, attributes (only if
-     * --extra-attributes is set) and members. If false, no data from relations
-     * is needed.
-     */
-    bool full_relations = false;
-};
-
 /**
  * Structure for storing command-line and other options
  */
@@ -81,7 +46,8 @@ struct options_t
 {
     command_t command = command_t::process;
 
-    std::string conninfo; ///< connection info for database
+    /// Parameters for initializing database connections
+    connection_params_t connection_params;
 
     std::string prefix{"planet_osm"};         ///< prefix for table names
     bool prefix_is_set = false;
@@ -117,7 +83,7 @@ struct options_t
     /// File name to output expired tiles list to
     std::string expire_tiles_filename{"dirty_tiles"};
 
-    std::string output_backend{"pgsql"};
+    std::string output_backend;
     std::string input_format; ///< input file format (default: autodetect)
 
     osmium::Box bbox;
@@ -149,8 +115,13 @@ struct options_t
      */
     uint8_t way_node_index_id_shift = 5;
 
-    /// Database format (0=unknown/no database middle, 1=legacy, 2=new)
-    uint8_t middle_database_format = 1;
+    /**
+     * Middle database format:
+     * 0 = non-slim mode, no database middle (ram middle)
+     * 1 = slim mode, legacy database format
+     * 2 = slim mode, new database format
+     */
+    uint8_t middle_database_format = 0;
 
     /**
      * Should nodes (with tags) be stored in the middle? If no flat node file
@@ -187,9 +158,6 @@ struct options_t
     bool parallel_indexing = true;
     bool create = false;
     bool pass_prompt = false;
-
-    bool output_backend_set = false;
-    bool style_set = false;
 }; // struct options_t
 
 #endif // OSM2PGSQL_OPTIONS_HPP
