@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -86,7 +86,7 @@ static file_info run(options_t const &options)
 
 static void check_db(options_t const &options)
 {
-    pg_conn_t const db_connection{options.conninfo};
+    pg_conn_t const db_connection{options.connection_params, "check"};
 
     init_database_capabilities(db_connection);
 
@@ -256,7 +256,7 @@ static void check_output(properties_t const &properties, options_t *options)
 {
     auto const output = properties.get_string("output", "pgsql");
 
-    if (!options->output_backend_set) {
+    if (options->output_backend.empty()) {
         options->output_backend = output;
         log_info("Using output '{}' (same as on import).", output);
         return;
@@ -334,9 +334,13 @@ static void check_for_nodes_table(options_t const &options)
     }
 }
 
-static void check_and_set_style(options_t *options)
+static void set_option_defaults(options_t *options)
 {
-    if (!options->style_set) {
+    if (options->output_backend.empty()) {
+        options->output_backend = "pgsql";
+    }
+
+    if (options->style.empty()) {
         if (options->output_backend == "flex" ||
             options->output_backend == "gazetteer") {
             throw std::runtime_error{"You have to set the config file "
@@ -348,11 +352,10 @@ static void check_and_set_style(options_t *options)
     }
 }
 
+// NOLINTNEXTLINE(bugprone-exception-escape)
 int main(int argc, char *argv[])
 {
     try {
-        log_info("osm2pgsql version {}", get_osm2pgsql_version());
-
         auto options = parse_command_line(argc, argv);
 
         if (options.command == command_t::help) {
@@ -369,12 +372,13 @@ int main(int argc, char *argv[])
 
         check_db(options);
 
-        properties_t properties{options.conninfo, options.middle_dbschema};
+        properties_t properties{options.connection_params,
+                                options.middle_dbschema};
         if (options.append) {
             if (properties.load()) {
                 check_and_update_properties(&properties, &options);
             } else {
-                check_and_set_style(&options);
+                set_option_defaults(&options);
                 check_for_nodes_table(options);
             }
 
@@ -392,7 +396,7 @@ int main(int argc, char *argv[])
                 }
             }
         } else {
-            check_and_set_style(&options);
+            set_option_defaults(&options);
             store_properties(&properties, options);
             auto const finfo = run(options);
             store_data_properties(&properties, finfo);
@@ -404,6 +408,9 @@ int main(int argc, char *argv[])
     } catch (std::exception const &e) {
         log_error("{}", e.what());
         return 1;
+    } catch (...) {
+        log_error("Unknown exception.");
+        return 1;
     }
 
     return 0;
