@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -48,6 +48,9 @@
 #include <stdexcept>
 #include <string>
 
+// Mutex used to coordinate access to Lua code
+static std::mutex lua_mutex;
+
 // Lua can't call functions on C++ objects directly. This macro defines simple
 // C "trampoline" functions which are called from Lua which get the current
 // context (the output_flex_t object) and call the respective function on the
@@ -584,6 +587,13 @@ int output_flex_t::table_add_row()
     }
     lua_remove(lua_state(), 1);
 
+    if (m_add_row_has_never_been_called) {
+        m_add_row_has_never_been_called = false;
+        log_warn("The add_row() function is deprecated. Please read");
+        log_warn("https://osm2pgsql.org/doc/tutorials/"
+                 "switching-from-add-row-to-insert/");
+    }
+
     if (m_calling_context == calling_context::process_node) {
         if (!table.matches_type(osmium::item_type::node)) {
             throw fmt_error("Trying to add node to table '{}'.", table.name());
@@ -895,9 +905,6 @@ int output_flex_t::expire_output_table()
 void output_flex_t::call_lua_function(prepared_lua_function_t func,
                                       osmium::OSMObject const &object)
 {
-    static std::mutex lua_mutex;
-    std::lock_guard<std::mutex> const guard{lua_mutex};
-
     m_calling_context = func.context();
 
     lua_pushvalue(lua_state(), func.index()); // the function to call
@@ -914,6 +921,13 @@ void output_flex_t::call_lua_function(prepared_lua_function_t func,
     m_calling_context = calling_context::main;
 }
 
+void output_flex_t::get_mutex_and_call_lua_function(
+    prepared_lua_function_t func, osmium::OSMObject const &object)
+{
+    std::lock_guard<std::mutex> const guard{lua_mutex};
+    call_lua_function(func, object);
+}
+
 void output_flex_t::pending_way(osmid_t id)
 {
     if (!m_process_way) {
@@ -926,7 +940,7 @@ void output_flex_t::pending_way(osmid_t id)
 
     way_delete(id);
 
-    call_lua_function(m_process_way, m_way_cache.get());
+    get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
 }
 
 void output_flex_t::select_relation_members()
@@ -935,6 +949,9 @@ void output_flex_t::select_relation_members()
         return;
     }
 
+    // We can not use get_mutex_and_call_lua_function() here, because we need
+    // the mutex to stick around as long as we are looking at the Lua stack.
+    std::lock_guard<std::mutex> const guard{lua_mutex};
     call_lua_function(m_select_relation_members, m_relation_cache.get());
 
     // If the function returned nil there is nothing to be marked.
@@ -1014,7 +1031,8 @@ void output_flex_t::pending_relation(osmid_t id)
     delete_from_tables(osmium::item_type::relation, id);
 
     if (m_process_relation) {
-        call_lua_function(m_process_relation, m_relation_cache.get());
+        get_mutex_and_call_lua_function(m_process_relation,
+                                        m_relation_cache.get());
     }
 }
 
@@ -1029,7 +1047,7 @@ void output_flex_t::pending_relation_stage1c(osmid_t id)
     }
 
     m_disable_add_row = true;
-    call_lua_function(m_process_relation, m_relation_cache.get());
+    get_mutex_and_call_lua_function(m_process_relation, m_relation_cache.get());
     m_disable_add_row = false;
 }
 
@@ -1058,7 +1076,10 @@ void output_flex_t::stop()
 {
     for (auto &table : m_table_connections) {
         table.task_set(thread_pool().submit([&]() {
-            table.stop(get_options()->slim && !get_options()->droptemp,
+            pg_conn_t const db_connection{get_options()->connection_params,
+                                          "out.flex.stop"};
+            table.stop(db_connection,
+                       get_options()->slim && !get_options()->droptemp,
                        get_options()->append);
         }));
     }
@@ -1068,8 +1089,9 @@ void output_flex_t::stop()
         if (!m_expire_tiles[i].empty()) {
             auto const &eo = (*m_expire_outputs)[i];
 
-            std::size_t const count = eo.output(m_expire_tiles[i].get_tiles(),
-                                                get_options()->conninfo);
+            std::size_t const count =
+                eo.output(m_expire_tiles[i].get_tiles(),
+                          get_options()->connection_params);
 
             log_info("Wrote {} entries to expire output [{}].", count, i);
         }
@@ -1104,7 +1126,7 @@ void output_flex_t::node_add(osmium::Node const &node)
     }
 
     m_context_node = &node;
-    call_lua_function(m_process_node, node);
+    get_mutex_and_call_lua_function(m_process_node, node);
     m_context_node = nullptr;
 }
 
@@ -1117,7 +1139,7 @@ void output_flex_t::way_add(osmium::Way *way)
     }
 
     m_way_cache.init(way);
-    call_lua_function(m_process_way, m_way_cache.get());
+    get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
 }
 
 void output_flex_t::relation_add(osmium::Relation const &relation)
@@ -1128,17 +1150,19 @@ void output_flex_t::relation_add(osmium::Relation const &relation)
 
     m_relation_cache.init(relation);
     select_relation_members();
-    call_lua_function(m_process_relation, relation);
+    get_mutex_and_call_lua_function(m_process_relation, relation);
 }
 
 void output_flex_t::delete_from_table(table_connection_t *table_connection,
+                                      pg_conn_t const &db_connection,
                                       osmium::item_type type, osmid_t osm_id)
 {
     assert(table_connection);
     auto const id = table_connection->table().map_id(type, osm_id);
 
     if (table_connection->table().has_columns_with_expire()) {
-        auto const result = table_connection->get_geoms_by_id(type, id);
+        auto const result =
+            table_connection->get_geoms_by_id(db_connection, type, id);
         auto const num_tuples = result.num_tuples();
         if (num_tuples > 0) {
             int col = 0;
@@ -1161,7 +1185,7 @@ void output_flex_t::delete_from_tables(osmium::item_type type, osmid_t osm_id)
 {
     for (auto &table : m_table_connections) {
         if (table.table().matches_type(type) && table.table().has_id_column()) {
-            delete_from_table(&table, type, osm_id);
+            delete_from_table(&table, m_db_connection, type, osm_id);
         }
     }
 }
@@ -1206,14 +1230,13 @@ void output_flex_t::relation_modify(osmium::Relation const &rel)
 void output_flex_t::start()
 {
     for (auto &table : m_table_connections) {
-        table.connect(get_options()->conninfo);
-        table.start(get_options()->append);
+        table.start(m_db_connection, get_options()->append);
     }
 }
 
 static void
 create_expire_tables(std::vector<expire_output_t> const &expire_outputs,
-                     std::string const &conninfo)
+                     connection_params_t const &connection_params)
 {
     if (std::all_of(expire_outputs.begin(), expire_outputs.end(),
                     [](auto const &expire_output) {
@@ -1222,7 +1245,7 @@ create_expire_tables(std::vector<expire_output_t> const &expire_outputs,
         return;
     }
 
-    pg_conn_t const connection{conninfo};
+    pg_conn_t const connection{connection_params, "out.flex.expire"};
     for (auto const &expire_output : expire_outputs) {
         if (!expire_output.table().empty()) {
             expire_output.create_output_table(connection);
@@ -1235,6 +1258,7 @@ output_flex_t::output_flex_t(output_flex_t const *other,
                              std::shared_ptr<db_copy_thread_t> copy_thread)
 : output_t(other, std::move(mid)), m_tables(other->m_tables),
   m_expire_outputs(other->m_expire_outputs),
+  m_db_connection(get_options()->connection_params, "out.flex.thread"),
   m_stage2_way_ids(other->m_stage2_way_ids),
   m_copy_thread(std::move(copy_thread)), m_lua_state(other->m_lua_state),
   m_process_node(other->m_process_node), m_process_way(other->m_process_way),
@@ -1243,8 +1267,7 @@ output_flex_t::output_flex_t(output_flex_t const *other,
 {
     for (auto &table : *m_tables) {
         auto &tc = m_table_connections.emplace_back(&table, m_copy_thread);
-        tc.connect(get_options()->conninfo);
-        tc.prepare();
+        tc.prepare(m_db_connection);
     }
 
     for (auto &expire_output : *m_expire_outputs) {
@@ -1264,7 +1287,8 @@ output_flex_t::output_flex_t(std::shared_ptr<middle_query_t> const &mid,
                              std::shared_ptr<thread_pool_t> thread_pool,
                              options_t const &options)
 : output_t(mid, std::move(thread_pool), options),
-  m_copy_thread(std::make_shared<db_copy_thread_t>(options.conninfo))
+  m_db_connection(get_options()->connection_params, "out.flex.main"),
+  m_copy_thread(std::make_shared<db_copy_thread_t>(options.connection_params))
 {
     init_lua(options.style);
 
@@ -1313,7 +1337,7 @@ output_flex_t::output_flex_t(std::shared_ptr<middle_query_t> const &mid,
                                     reprojection::create_projection(3857));
     }
 
-    create_expire_tables(*m_expire_outputs, get_options()->conninfo);
+    create_expire_tables(*m_expire_outputs, get_options()->connection_params);
 }
 
 /**
@@ -1490,8 +1514,8 @@ void output_flex_t::reprocess_marked()
         for (auto &table : m_table_connections) {
             if (table.table().matches_type(osmium::item_type::way) &&
                 table.table().has_id_column()) {
-                table.analyze();
-                table.create_id_index();
+                table.analyze(m_db_connection);
+                table.create_id_index(m_db_connection);
             }
         }
 
@@ -1518,7 +1542,7 @@ void output_flex_t::reprocess_marked()
         }
         way_delete(id);
         if (m_process_way) {
-            call_lua_function(m_process_way, m_way_cache.get());
+            get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
         }
     }
 
