@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -185,12 +185,15 @@ private:
 
     /**
      * Call a Lua function that was "prepared" earlier with the OSMObject
-     * as its only parameter. Uses a mutex internally to make access to the
-     * Lua environment thread safe.
+     * as its only parameter.
      */
     void call_lua_function(prepared_lua_function_t func,
                            osmium::OSMObject const &object);
 
+    /// Aquire the lua_mutex and the call `call_lua_function()`.
+    void get_mutex_and_call_lua_function(prepared_lua_function_t func,
+                                         osmium::OSMObject const &object);
+
     void init_lua(std::string const &filename);
 
     // Get the flex table that is as first parameter on the Lua stack.
@@ -221,7 +224,9 @@ private:
     void add_row(table_connection_t *table_connection, OBJECT const &object);
 
     void delete_from_table(table_connection_t *table_connection,
+                           pg_conn_t const &db_connection,
                            osmium::item_type type, osmid_t osm_id);
+
     void delete_from_tables(osmium::item_type type, osmid_t osm_id);
 
     lua_State *lua_state() noexcept { return m_lua_state.get(); }
@@ -285,6 +290,9 @@ private:
 
     std::vector<table_connection_t> m_table_connections;
 
+    /// The connection to the database server.
+    pg_conn_t m_db_connection;
+
     // This is shared between all clones of the output and must only be
     // accessed while protected using the lua_mutex.
     std::shared_ptr<idset_t> m_stage2_way_ids = std::make_shared<idset_t>();
@@ -313,6 +321,8 @@ private:
      * add_row() command.
      */
     bool m_disable_add_row = false;
+
+    bool m_add_row_has_never_been_called = true;
 };
 
 #endif // OSM2PGSQL_OUTPUT_FLEX_HPP
