@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -49,7 +49,7 @@ void output_gazetteer_t::start()
     if (!get_options()->append) {
         int const srid = get_options()->projection->target_srs();
 
-        pg_conn_t const conn{get_options()->conninfo};
+        pg_conn_t const conn{get_options()->connection_params, "out.gazetteer"};
 
         /* Drop any existing table */
         conn.exec("DROP TABLE IF EXISTS place CASCADE");
