@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -27,12 +27,12 @@ class pg_result_t;
  */
 idlist_t get_ids_from_result(pg_result_t const &result);
 
-void create_geom_check_trigger(pg_conn_t *db_connection,
+void create_geom_check_trigger(pg_conn_t const &db_connection,
                                std::string const &schema,
                                std::string const &table,
                                std::string const &condition);
 
-void drop_geom_check_trigger(pg_conn_t *db_connection,
+void drop_geom_check_trigger(pg_conn_t const &db_connection,
                              std::string const &schema,
                              std::string const &table);
 
