@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -28,21 +28,46 @@ std::size_t pg_result_t::affected_rows() const noexcept
 
 std::atomic<std::uint32_t> pg_conn_t::connection_id{0};
 
-pg_conn_t::pg_conn_t(std::string const &conninfo)
-: m_conn(PQconnectdb(conninfo.c_str())),
-  m_connection_id(connection_id.fetch_add(1))
+static PGconn *open_connection(connection_params_t const &connection_params,
+                               std::string_view context, std::uint32_t id)
 {
-    if (!m_conn) {
-        throw std::runtime_error{"Connecting to database failed."};
+    std::vector<char const *> keywords;
+    std::vector<char const *> values;
+
+    for (auto const &[k, v] : connection_params) {
+        keywords.push_back(k.c_str());
+        values.push_back(v.c_str());
     }
+
+    std::string const app_name{fmt::format("osm2pgsql.{}/C{}", context, id)};
+    keywords.push_back("fallback_application_name");
+    values.push_back(app_name.c_str());
+
+    keywords.push_back(nullptr);
+    values.push_back(nullptr);
+
+    return PQconnectdbParams(keywords.data(), values.data(), 1);
+}
+
+pg_conn_t::pg_conn_t(connection_params_t const &connection_params,
+                     std::string_view context)
+: m_connection_id(connection_id.fetch_add(1))
+{
+    m_conn.reset(open_connection(connection_params, context, m_connection_id));
+
+    if (!m_conn) {
+        throw fmt_error("Connecting to database failed (context={}).", context);
+    }
+
     if (PQstatus(m_conn.get()) != CONNECTION_OK) {
-        throw fmt_error("Connecting to database failed: {}.", error_msg());
+        throw fmt_error("Connecting to database failed (context={}): {}.",
+                        context, error_msg());
     }
 
     if (get_logger().log_sql()) {
         auto const results = exec("SELECT pg_backend_pid()");
-        log_sql("(C{}) New database connection (backend_pid={})",
-                m_connection_id, results.get(0, 0));
+        log_sql("(C{}) New database connection (context={}, backend_pid={})",
+                m_connection_id, context, results.get(0, 0));
     }
 
     // PostgreSQL sends notices in many different contexts which aren't that
@@ -50,6 +75,11 @@ pg_conn_t::pg_conn_t(std::string const &conninfo)
     if (!get_logger().debug_enabled()) {
         exec("SET client_min_messages = WARNING");
     }
+
+    // Disable synchronous_commit on all connections. For some connections it
+    // might not matter, especially if they read only, but then it doesn't
+    // hurt either.
+    exec("SET synchronous_commit = off");
 }
 
 void pg_conn_t::close()
@@ -91,25 +121,24 @@ pg_result_t pg_conn_t::exec(std::string const &sql) const
     return exec(sql.c_str());
 }
 
-void pg_conn_t::copy_start(char const *sql) const
+void pg_conn_t::copy_start(std::string_view sql) const
 {
     assert(m_conn);
 
     log_sql("(C{}) {}", m_connection_id, sql);
-    pg_result_t const res{PQexec(m_conn.get(), sql)};
+    pg_result_t const res{PQexec(m_conn.get(), sql.data())};
     if (res.status() != PGRES_COPY_IN) {
         throw fmt_error("Database error on COPY: {}", error_msg());
     }
 }
 
-void pg_conn_t::copy_send(std::string const &data,
-                          std::string const &context) const
+void pg_conn_t::copy_send(std::string_view data, std::string_view context) const
 {
     assert(m_conn);
 
     log_sql_data("(C{}) Copy data to '{}':\n{}", m_connection_id, context,
                  data);
-    int const r = PQputCopyData(m_conn.get(), data.c_str(), (int)data.size());
+    int const r = PQputCopyData(m_conn.get(), data.data(), (int)data.size());
 
     switch (r) {
     case 0: // need to wait for write ready
@@ -127,14 +156,14 @@ void pg_conn_t::copy_send(std::string const &data,
     if (data.size() < 1100) {
         log_error("Data: {}", data);
     } else {
-        log_error("Data: {}\n...\n{}", std::string(data, 0, 500),
-                  std::string(data, data.size() - 500));
+        log_error("Data: {}\n...\n{}", data.substr(0, 500),
+                  data.substr(data.size() - 500, 500));
     }
 
     throw std::runtime_error{"COPYing data to Postgresql."};
 }
 
-void pg_conn_t::copy_end(std::string const &context) const
+void pg_conn_t::copy_end(std::string_view context) const
 {
     assert(m_conn);
 
