@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -17,6 +17,7 @@
  */
 
 #include "format.hpp"
+#include "pgsql-params.hpp"
 
 #include <libpq-fe.h>
 
@@ -151,7 +152,8 @@ public:
 class pg_conn_t
 {
 public:
-    explicit pg_conn_t(std::string const &conninfo);
+    explicit pg_conn_t(connection_params_t const &connection_params,
+                       std::string_view context);
 
     /**
      * Run the specified SQL command.
@@ -215,9 +217,9 @@ public:
      */
     void set_config(char const *setting, char const *value) const;
 
-    void copy_start(char const *sql) const;
-    void copy_send(std::string const &data, std::string const &context) const;
-    void copy_end(std::string const &context) const;
+    void copy_start(std::string_view sql) const;
+    void copy_send(std::string_view data, std::string_view context) const;
+    void copy_end(std::string_view context) const;
 
     /// Return the latest generated error message on this connection.
     char const *error_msg() const noexcept;
