@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -45,8 +45,8 @@ table_t::table_t(std::string const &name, std::string type, columns_t columns,
 
 table_t::table_t(table_t const &other,
                  std::shared_ptr<db_copy_thread_t> const &copy_thread)
-: m_conninfo(other.m_conninfo), m_target(other.m_target), m_type(other.m_type),
-  m_srid(other.m_srid), m_append(other.m_append),
+: m_connection_params(other.m_connection_params), m_target(other.m_target),
+  m_type(other.m_type), m_srid(other.m_srid), m_append(other.m_append),
   m_hstore_mode(other.m_hstore_mode), m_columns(other.m_columns),
   m_hstore_columns(other.m_hstore_columns), m_table_space(other.m_table_space),
   m_copy(copy_thread)
@@ -66,19 +66,18 @@ void table_t::sync() { m_copy.sync(); }
 
 void table_t::connect()
 {
-    m_sql_conn = std::make_unique<pg_conn_t>(m_conninfo);
-    //let commits happen faster by delaying when they actually occur
-    m_sql_conn->exec("SET synchronous_commit = off");
+    m_sql_conn = std::make_unique<pg_conn_t>(m_connection_params, "out.pgsql");
 }
 
-void table_t::start(std::string const &conninfo, std::string const &table_space)
+void table_t::start(connection_params_t const &connection_params,
+                    std::string const &table_space)
 {
     if (m_sql_conn) {
         throw fmt_error("{} cannot start, its already started.",
                         m_target->name());
     }
 
-    m_conninfo = conninfo;
+    m_connection_params = connection_params;
     m_table_space = tablespace_clause(table_space);
 
     connect();
@@ -132,7 +131,7 @@ void table_t::start(std::string const &conninfo, std::string const &table_space)
         m_sql_conn->exec(sql);
 
         if (m_srid != "4326") {
-            create_geom_check_trigger(m_sql_conn.get(), m_target->schema(),
+            create_geom_check_trigger(*m_sql_conn, m_target->schema(),
                                       m_target->name(), "ST_IsValid(NEW.way)");
         }
     }
@@ -188,7 +187,7 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
 
     if (!m_append) {
         if (m_srid != "4326") {
-            drop_geom_check_trigger(m_sql_conn.get(), m_target->schema(),
+            drop_geom_check_trigger(*m_sql_conn, m_target->schema(),
                                     m_target->name());
         }
 
@@ -237,7 +236,7 @@ void table_t::stop(bool updateable, bool enable_hstore_index,
             m_sql_conn->exec("CREATE INDEX ON {} USING BTREE (osm_id) {}",
                              qual_name, tablespace_clause(table_space_index));
             if (m_srid != "4326") {
-                create_geom_check_trigger(m_sql_conn.get(), m_target->schema(),
+                create_geom_check_trigger(*m_sql_conn, m_target->schema(),
                                           m_target->name(),
                                           "ST_IsValid(NEW.way)");
             }
