@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -14,6 +14,7 @@
 #include <cstdlib>
 #include <stdexcept>
 #include <string>
+#include <string_view>
 
 #include "format.hpp"
 #include "options.hpp"
@@ -30,14 +31,16 @@
 #include <unistd.h>
 #endif
 
-namespace testing {
 /// Helper classes for postgres connections
-namespace pg {
+namespace testing::pg {
 
 class conn_t : public pg_conn_t
 {
 public:
-    conn_t(std::string const &conninfo) : pg_conn_t(conninfo) {}
+    conn_t(connection_params_t const &connection_params)
+    : pg_conn_t(connection_params, "test")
+    {
+    }
 
     std::string result_as_string(std::string const &cmd) const
     {
@@ -76,7 +79,8 @@ public:
         return res;
     }
 
-    int get_count(char const *table_name, std::string const &where = "") const
+    int get_count(std::string_view table_name,
+                  std::string_view where = "") const
     {
         auto const query =
             fmt::format("SELECT count(*) FROM {} {} {}", table_name,
@@ -85,7 +89,7 @@ public:
         return result_as_int(query);
     }
 
-    void require_has_table(char const *table_name) const
+    void require_has_table(std::string_view table_name) const
     {
         auto const where = fmt::format("oid = '{}'::regclass", table_name);
 
@@ -99,7 +103,9 @@ public:
     tempdb_t() noexcept
     {
         try {
-            conn_t conn{"dbname=postgres"};
+            connection_params_t connection_params;
+            connection_params.set("dbname", "postgres");
+            conn_t conn{connection_params};
 
             m_db_name =
                 fmt::format("osm2pgsql-test-{}-{}", getpid(), time(nullptr));
@@ -137,7 +143,9 @@ public:
                 return;
             }
             try {
-                conn_t conn{"dbname=postgres"};
+                connection_params_t connection_params;
+                connection_params.set("dbname", "postgres");
+                conn_t conn{connection_params};
                 conn.exec(R"(DROP DATABASE IF EXISTS "{}")", m_db_name);
             } catch (...) {
                 fprintf(stderr, "DROP DATABASE failed. Ignored.\n");
@@ -145,15 +153,18 @@ public:
         }
     }
 
-    conn_t connect() const { return conn_t{conninfo()}; }
+    conn_t connect() const { return conn_t{connection_params()}; }
 
-    std::string conninfo() const { return "dbname=" + m_db_name; }
+    connection_params_t connection_params() const {
+        connection_params_t params;
+        params.set("dbname", m_db_name);
+        return params;
+    }
 
 private:
     std::string m_db_name;
 };
 
-} // namespace pg
-} // namespace testing
+} // namespace testing::pg
 
 #endif // OSM2PGSQL_TESTS_COMMON_PG_HPP
