@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2023 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2024 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -11,7 +11,7 @@
 
 #include <vector>
 
-#include "options.hpp"
+#include "command-line-parser.hpp"
 #include "taginfo-impl.hpp"
 #include "tagtransform.hpp"
 
@@ -21,22 +21,27 @@ static void bad_opt(std::vector<char const *> opts, char const *msg)
 {
     opts.insert(opts.begin(), "osm2pgsql");
     opts.push_back(TEST_PBF);
-    REQUIRE_THROWS_WITH(options_t((int)opts.size(), (char **)opts.data()),
-                        Catch::Matchers::Contains(msg));
+
+    REQUIRE_THROWS_WITH(
+        parse_command_line((int)opts.size(), (char **)opts.data()),
+        Catch::Matchers::Contains(msg));
 }
 
 static options_t opt(std::vector<char const *> opts)
 {
     opts.insert(opts.begin(), "osm2pgsql");
     opts.push_back(TEST_PBF);
-    return {(int)opts.size(), (char **)opts.data()};
+
+    return parse_command_line((int)opts.size(), (char **)opts.data());
 }
 
 TEST_CASE("Insufficient arguments", "[NoDB]")
 {
-    std::vector<char const *> opts = {"osm2pgsql", "-a", "-c", "--slim"};
-    REQUIRE_THROWS_WITH(options_t((int)opts.size(), (char **)opts.data()),
-                        Catch::Matchers::Contains("Missing input"));
+    std::vector<char const *> opts = {"osm2pgsql", "-c", "--slim"};
+
+    REQUIRE_THROWS_WITH(
+        parse_command_line((int)opts.size(), (char **)opts.data()),
+        Catch::Matchers::Contains("Missing input"));
 }
 
 TEST_CASE("Incompatible arguments", "[NoDB]")
@@ -45,7 +50,7 @@ TEST_CASE("Incompatible arguments", "[NoDB]")
 
     bad_opt({"--drop"}, "drop only makes sense with");
 
-    bad_opt({"-j", "-k"}, "You can not specify both");
+    bad_opt({"-j", "-k"}, "--hstore excludes --hstore-all");
 
     bad_opt({"-a"}, "--append can only be used with slim mode");
 }
@@ -62,10 +67,8 @@ TEST_CASE("Middle selection", "[NoDB]")
 TEST_CASE("Lua styles", "[NoDB]")
 {
 #ifdef HAVE_LUA
-    auto options = opt({"--tag-transform-script", "non_existing.lua"});
-    export_list const exlist;
-    REQUIRE_THROWS_WITH(tagtransform_t::make_tagtransform(&options, exlist),
-                        Catch::Matchers::Contains("No such file or directory"));
+    REQUIRE_THROWS_WITH(opt({"--tag-transform-script", "non_existing.lua"}),
+                        Catch::Matchers::Contains("File does not exist"));
 #endif
 }
 
@@ -175,7 +178,7 @@ TEST_CASE("Parsing log-level", "[NoDB]")
 
 TEST_CASE("Parsing log-level fails for unknown level", "[NoDB]")
 {
-    bad_opt({"--log-level", "foo"}, "Unknown value for --log-level option: ");
+    bad_opt({"--log-level", "foo"}, "--log-level: foo not in");
 }
 
 TEST_CASE("Parsing log-progress", "[NoDB]")
